@@ -51,7 +51,7 @@ use std::ffi::CString;
 use std::panic::catch_unwind;
 use std::path::PathBuf;
 use std::ptr::null_mut;
-use std::str::from_utf8;
+use std::str::{from_utf8, FromStr};
 
 use bytes::Bytes;
 use chrono::Local;
@@ -63,19 +63,32 @@ use maplit::*;
 use onig::Regex;
 use rand::prelude::*;
 use serde_json::json;
+use serde_json::Value;
 use uuid::Uuid;
 
 use pact_matching::logging::fetch_buffer_contents;
-use pact_matching::models::{HttpPart, Pact, RequestResponseInteraction};
-use pact_matching::models::generators::Generators;
-use pact_matching::models::matchingrules::{MatchingRule, RuleLogic};
-use pact_matching::models::matchingrules::MatchingRules;
+use pact_matching::models::{Pact, RequestResponseInteraction};
 use pact_matching::models::message::Message;
+// Export all the FFI logging functions from pact_matching_ffi
+pub use pact_matching_ffi::log::{
+  fetch_log_buffer,
+  log_to_buffer,
+  log_to_file,
+  log_to_stderr,
+  log_to_stdout,
+  logger_apply,
+  logger_attach_sink,
+  logger_init
+};
 use pact_mock_server::{MANAGER, MockServerError, tls::TlsConfigBuilder, WritePactFileErr};
 use pact_mock_server::server_manager::ServerManager;
 use pact_models::bodies::OptionalBody::{Null, Present};
 use pact_models::bodies::OptionalBody;
 use pact_models::content_types::ContentType;
+use pact_models::generators::Generators;
+use pact_models::http_parts::HttpPart;
+use pact_models::json_utils::json_to_string;
+use pact_models::matchingrules::{MatchingRule, MatchingRules, RuleLogic};
 use pact_models::PactSpecification;
 use pact_models::provider_states::ProviderState;
 use pact_models::time_utils::{parse_pattern, to_chrono_pattern};
@@ -84,18 +97,6 @@ use crate::bodies::{empty_multipart_body, file_as_multipart_body, MultipartBody,
 use crate::bodies::process_object;
 use crate::handles::InteractionPart;
 
-// Export all the FFI logging functions from pact_matching_ffi
-pub use pact_matching_ffi::log::{
-  logger_apply,
-  logger_attach_sink,
-  logger_init,
-  fetch_log_buffer,
-  log_to_stdout,
-  log_to_stderr,
-  log_to_file,
-  log_to_buffer
-};
-
 pub mod handles;
 pub mod bodies;
 
@@ -133,6 +134,57 @@ pub unsafe extern fn init(log_env_var: *const c_char) {
   builder.try_init().unwrap_or(());
 }
 
+/// Initialises logging, and sets the log level explicitly.
+///
+/// # Safety
+///
+/// Exported functions are inherently unsafe.
+#[no_mangle]
+pub unsafe extern "C" fn init_with_log_level(level: *const c_char) {
+  let mut builder = Builder::from_default_env();
+  let log_level = log_level_from_c_char(level);
+
+  builder.filter_level(log_level.to_level_filter());
+  builder.try_init().unwrap_or(());
+}
+
+/// Log using the shared core logging facility.
+///
+/// This is useful for callers to have a single set of logs.
+///
+/// * `source` - String. The source of the log, such as the class or caller framework to
+///                      disambiguate log lines from the rust logging (e.g. pact_go)
+/// * `log_level` - String. One of TRACE, DEBUG, INFO, WARN, ERROR
+/// * `message` - Message to log
+///
+/// Exported functions are inherently unsafe.
+#[no_mangle]
+pub unsafe extern "C" fn log_message(source: *const c_char, log_level: *const c_char, message: *const c_char) {
+  let target = convert_cstr("target", source).unwrap_or("client");
+
+  if !message.is_null() {
+    match convert_cstr("message", message) {
+      Some(message) => log!(
+        target: target,
+        log_level_from_c_char(log_level),
+        "{}",
+        message
+      ),
+      None => (),
+    }
+  }
+}
+
+unsafe fn log_level_from_c_char(log_level: *const c_char) -> log::Level {
+  if !log_level.is_null() {
+    let level = convert_cstr("log_level", log_level).unwrap_or("INFO");
+
+    log::Level::from_str(level).unwrap_or(log::Level::Info)
+  } else {
+    log::Level::Info
+  }
+}
+
 /// External interface to create a mock server. A pointer to the pact JSON as a C string is passed in,
 /// as well as the port for the mock server to run on. A value of 0 for the port will result in a
 /// port being allocated by the operating system. The port of the mock server is returned.
@@ -663,7 +715,7 @@ fn from_integration_json(rules: &mut MatchingRules, generators: &mut Generators,
       serde_json::Value::Object(ref map) => {
         let json: serde_json::Value = process_object(map, category, generators, path, false, false);
         // These are simple JSON primitives (strings), so we must unescape them
-        json.as_str().unwrap_or_default().to_string()
+        json_to_string(&json)
       },
       _ => value.to_string()
     },
@@ -1029,12 +1081,13 @@ pub unsafe extern fn free_string(s: *mut c_char) {
 /// * `part` - Request or response part.
 /// * `content_type` - Expected content type.
 /// * `body` - example body contents in bytes
+/// * `size` - number of bytes in the body
 #[no_mangle]
 pub extern fn with_binary_file(
   interaction: handles::InteractionHandle,
   part: InteractionPart,
   content_type: *const c_char,
-  body: *const c_char ,
+  body: *const u8 ,
   size: size_t
 ) -> bool {
   let content_type_header = "Content-Type".to_string();
@@ -1138,13 +1191,13 @@ pub extern fn with_multipart_file(
   }
 }
 
-fn convert_ptr_to_body(body: *const c_char, size: size_t) -> OptionalBody {
+fn convert_ptr_to_body(body: *const u8, size: size_t) -> OptionalBody {
   if body.is_null() {
     OptionalBody::Null
   } else if size == 0 {
     OptionalBody::Empty
   } else {
-    OptionalBody::Present(Bytes::from(unsafe { std::slice::from_raw_parts(body as *const u8, size) }), None)
+    OptionalBody::Present(Bytes::from(unsafe { std::slice::from_raw_parts(body, size) }), None)
   }
 }
 
@@ -1256,27 +1309,31 @@ pub extern fn message_given_with_param(message: handles::MessageHandle, descript
 ///
 /// Accepts JSON, binary and other payload types. Binary data will be base64 encoded when serialised.
 ///
+/// Note: For text bodies (plain text, JSON or XML), you can pass in a C string (NULL terminated)
+/// and the size of the body is not required (it will be ignored). For binary bodies, you need to
+/// specify the number of bytes in the body.
+///
 /// * `content_type` - The content type of the body. Defaults to `text/plain`, supports JSON structures with matchers and binary data.
-/// * `body` - The body contents. For JSON payloads, matching rules can be embedded in the body.
+/// * `body` - The body contents as bytes. For text payloads (JSON, XML, etc.), a C string can be used and matching rules can be embedded in the body.
 /// * `content_type` - Expected content type (e.g. application/json, application/octet-stream)
-/// * `size` - number of bytes in the message to read
+/// * `size` - number of bytes in the message body to read. This is not required for text bodies (JSON, XML, etc.).
 #[no_mangle]
-pub extern fn message_with_contents(message: handles::MessageHandle, content_type: *const c_char, body: *const c_char, size: size_t) {
+pub extern fn message_with_contents(message: handles::MessageHandle, content_type: *const c_char, body: *const u8, size: size_t) {
   let content_type = convert_cstr("content_type", content_type).unwrap_or("text/plain");
 
   message.with_message(&|_, inner| {
     let content_type = ContentType::parse(content_type).ok();
 
     let body = if let Some(content_type) = content_type {
-      if content_type.is_json() {
-        let body = convert_cstr("body", body).unwrap_or_default();
+      if content_type.is_text() {
+        let body = convert_cstr("body", body as *const c_char).unwrap_or_default();
         let category = inner.matching_rules.add_category("body");
         OptionalBody::Present(Bytes::from(process_json(body.to_string(), category, &mut inner.generators)), Some(content_type))
       } else {
-        OptionalBody::Present(Bytes::from(unsafe { std::slice::from_raw_parts(body as *const u8, size) }), Some(content_type))
+        OptionalBody::Present(Bytes::from(unsafe { std::slice::from_raw_parts(body, size) }), Some(content_type))
       }
     } else {
-      OptionalBody::Present(Bytes::from(unsafe { std::slice::from_raw_parts(body as *const u8, size) }), None)
+      OptionalBody::Present(Bytes::from(unsafe { std::slice::from_raw_parts(body, size) }), None)
     };
 
     inner.contents = body;
@@ -1291,7 +1348,7 @@ pub extern fn message_with_contents(message: handles::MessageHandle, content_typ
 pub extern fn message_with_metadata(message: handles::MessageHandle, key: *const c_char, value: *const c_char) {
   if let Some(key) = convert_cstr("key", key) {
     let value = convert_cstr("value", value).unwrap_or_default();
-    message.with_message(&|_, inner| inner.metadata.insert(key.to_string(), value.to_string()));
+    message.with_message(&|_, inner| inner.metadata.insert(key.to_string(), Value::String(value.to_string())));
   }
 }
 
